from requests import Session
from requests import Response, JSONDecodeError
from urllib.parse import unquote_plus, quote_plus


class Authentication:

    @staticmethod
    def login(email: str, password: str) -> str:

        session: Session = Session()

        session.headers["user-agent"] = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 " \
                                        "(KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36"

        session.get(
            url="https://chat.openai.com/api/auth/session",
            headers={
                "accept": "*/*",
                "accept-language": "en-US,en;q=0.9",
                "content-type": "application/x-www-form-urlencoded",
                "sec-ch-ua": "\"Not_A Brand\";v=\"8\", \"Chromium\";v=\"120\", \"Google Chrome\";v=\"120\"",
                "sec-ch-ua-mobile": "?0",
                "sec-ch-ua-platform": "\"Windows\"",
                "sec-fetch-dest": "empty",
                "sec-fetch-mode": "cors",
                "sec-fetch-site": "same-origin",
                "Referer": "https://chat.openai.com/auth/login",
                "Referrer-Policy": "strict-origin-when-cross-origin"
            }
        )

        redirect_response: Response = session.post(
            url="https://chat.openai.com/api/auth/signin/auth0?prompt=login",
            headers={
                "accept": "*/*",
                "accept-language": "en-US,en;q=0.9",
                "content-type": "application/x-www-form-urlencoded",
                "sec-ch-ua": "\"Not_A Brand\";v=\"8\", \"Chromium\";v=\"120\", \"Google Chrome\";v=\"120\"",
                "sec-ch-ua-mobile": "?0",
                "sec-ch-ua-platform": "\"Windows\"",
                "sec-fetch-dest": "empty",
                "sec-fetch-mode": "cors",
                "sec-fetch-site": "same-origin",
                "Referer": "https://chat.openai.com/auth/login",
                "Referrer-Policy": "strict-origin-when-cross-origin"
            },
            data=f'callbackUrl=%2F&csrfToken='
                 f'{unquote_plus(session.cookies.get("__Host-next-auth.csrf-token")).split("|")[0]}'
                 f'&json=true'
        )

        try:
            redirect_response_json: dict = redirect_response.json()

            if "url" not in redirect_response_json:
                raise Exception("Key 'url' not in response JSON. Most probably it's a CSRF issue with this package.")

            # Get State
            state: str = session.get(
                redirect_response_json["url"],
                allow_redirects=False
            ).headers.get("location").split("state=")[-1]

            # Load Login Page for Cookies!
            session.get(
                url=f"https://auth0.openai.com/u/login/password?state={state}",
                headers={
                    "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,"
                              "image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
                    "accept-language": "en-US,en;q=0.9",
                    "cache-control": "no-cache",
                    "pragma": "no-cache",
                    "sec-ch-ua": "\"Not_A Brand\";v=\"8\", \"Chromium\";v=\"120\", \"Google Chrome\";v=\"120\"",
                    "sec-ch-ua-mobile": "?0",
                    "sec-ch-ua-platform": "\"Windows\"",
                    "sec-fetch-dest": "document",
                    "sec-fetch-mode": "navigate",
                    "sec-fetch-site": "same-origin",
                    "sec-fetch-user": "?1",
                    "upgrade-insecure-requests": "1",
                    "Referer": f"https://auth0.openai.com/u/login/identifier?state={state}",
                    "Referrer-Policy": "same-origin"
                }
            )

            # Authenticate
            auth_response: Response = session.post(
                url=f"https://auth0.openai.com/u/login/password?state={state}",
                headers={
                    "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,"
                              "image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
                    "accept-language": "en-US,en;q=0.9",
                    "cache-control": "max-age=0",
                    "content-type": "application/x-www-form-urlencoded",
                    "sec-ch-ua": "\"Not_A Brand\";v=\"8\", \"Chromium\";v=\"120\", \"Google Chrome\";v=\"120\"",
                    "sec-ch-ua-mobile": "?0",
                    "sec-ch-ua-platform": "\"Windows\"",
                    "sec-fetch-dest": "document",
                    "sec-fetch-mode": "navigate",
                    "sec-fetch-site": "same-origin",
                    "sec-fetch-user": "?1",
                    "upgrade-insecure-requests": "1",
                    "Referer": f"https://auth0.openai.com/u/login/password?state={state}",
                    "Referrer-Policy": "same-origin"
                },
                data=f"state={state}&username={quote_plus(email)}&password={quote_plus(password)}",
                allow_redirects=False
            )

            return auth_response.status_code

        except JSONDecodeError:
            raise Exception("HTTP response is not a valid JSON. Most probably it's a CSRF issue with this package.")



import requests
from requests import Session, Response, JSONDecodeError
import json
from uuid import uuid4


class Generative:

    session: Session

    email: str
    password: str

    session_token: str
    bearer_token: str
    logging: bool

    def __init__(
            self,
            session_token: str,
            history_and_training_enabled: bool = False,
            logging: bool = False
    ) -> None:

        self.logging = logging
        self.session_token = session_token
        self.history_and_training_enabled = history_and_training_enabled
        self.session: Session = requests.Session()

        self.session.headers["user-agent"] = "node"

        self.refresh_bearer_token()

    def refresh_bearer_token(self) -> None:

        response: Response = requests.get(
            url="https://chat.openai.com/api/auth/session",
            headers={
                "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
                              "(KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36",
                "accept": "*/*",
                "accept-language": "en-US",
                "content-type": "application/json",
                "sec-fetch-dest": "empty",
                "sec-fetch-mode": "cors",
                "sec-fetch-site": "same-origin",
                "cookie": f"__Secure-next-auth.session-token={self.session_token}",
                "Referer": "https://chat.openai.com",
                "Referrer-Policy": "strict-origin-when-cross-origin"
            }
        )

        try:
            response_json: dict = response.json()

            if "accessToken" not in response_json:
                raise Exception(
                    "Key 'accessToken' not in response JSON. Most probably you're not "
                    "logged in. Please check your credentials and try deleting the session file."
                )

            self.bearer_token = response_json["accessToken"]

        except JSONDecodeError:
            raise Exception(
                "HTTP Response is not a valid JSON. "
                "Please check your network connection or switch to another network."
            )

    def prompt(self, text: str) -> str:

        body: dict = {
            "action": "next",
            "arkose_token": "null",
            "conversation_mode": {
                "kind": "primary_assistant"
            },
            "force_paragen": False,
            "force_rate_limit": False,
            "history_and_training_disabled": not self.history_and_training_enabled,
            "messages": [{
                "metadata": {},
                "author": {
                    "role": "user"
                },
                "content": {
                    "content_type": "text",
                    "parts": [text]
                }
            }],
            "model": "text-davinci-002-render-sha",
            "parent_message_id": str(uuid4()),
            "timezone_offset_min": -330
        }

        response: Response = self.session.post(
            url="https://chat.openai.com/backend-api/conversation",
            headers={
                "accept": "text/event-stream",
                "accept-language": "en-US",
                "authorization": f"Bearer {self.bearer_token}",
                "content-type": "application/json",
                "sec-fetch-dest": "empty",
                "sec-fetch-mode": "cors",
                "sec-fetch-site": "same-origin",
                "Referer": "https://chat.openai.com/",
                "Referrer-Policy": "strict-origin-when-cross-origin"
            },
            data=json.dumps(body)
        )

        data: dict = {}

        if self.logging:
            with open("chatgpt-response.txt", "w") as file: file.write(response.text)

        for chunk in response.text.split("\n"):

            if chunk.startswith("data: {\"message\":"):

                try: data: dict = json.loads(chunk[6:])
                except JSONDecodeError:
                    raise Exception("Couldn't parse assistant's answer into a valid JSON. Please try another prompt.")

        if data["message"]["status"] != "finished_successfully":

            raise Exception(
                "Assistant's message was not finished successfully. Most probably there's an issue with this package."
            )

        if data["message"]["content"]["content_type"] != "text":

            raise Exception(
                "Didn't receive 'text' data type as assistant's response."
            )

        message_parts: list = data["message"]["content"]["parts"]

        return "".join(message_parts)



from distutils.core import setup
from pathlib import Path

version = "1.0.5"
long_description = (Path(__file__).parent / "README.md").read_text(encoding="utf-8")

setup(
    name="ProGPT",
    packages=["ProGPT"],
    version=version,
    license="MIT",
    description="❄️ Python Package To Access GPT-3.5 Free Model",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Deepak Soni",
    author_email="sonniiii@outlook.com",
    url="https://github.com/diezo/progpt",
    download_url=f"https://github.com/diezo/progpt/archive/refs/tags/v{version}.tar.gz",
    keywords=[
        "progpt",
        "chatgpt",
        "gpt-3.5",
        "python-gpt",
        "python-chatgpt",
        "ai",
        "generative-ai",
        "chatgpt-ai",
        "python-ai",
        "python-generative-ai"
    ],
    install_requires=[
        "requests",
        "selenium"
    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10"
    ]
)
